% doubleIntegratorIC: script for figures of double integrator initial conditions
% 
% This script generates three figures showing the initial conditions for
% the double integrator problem.  Specifically, the figures show:
%
%   1) The lower bounds for the hexagonal target set
%
%   2) The upper bounds for the hexagonal target set
%
%   3) The upper and lower bounds for the hexagonal target set, with the
%      region where there is a nonempty interval between the bounds
%      highlighted.

% Copyright 2009 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 9/01/09
% Subversion tags for version control purposes.
% $Date: 2011-04-11 10:47:05 -0700 (Mon, 11 Apr 2011) $
% $Id: doubleIntegratorIC.m 581 2011-04-11 17:47:05Z mitchell $

  %---------------------------------------------------------------------------
  % You will see many executable lines that are commented out.
  %   These are included to show some of the options available; modify
  %   the commenting to modify the behavior.
  
  %---------------------------------------------------------------------------
  %% Make sure we can see the kernel m-files.
  run('addPathToKernel');

  %---------------------------------------------------------------------------
  %% Create the grid.
  % Note that only the nonlinear variables need to be in the grid; in this
  % case, only one dimension.  We make the grid a little bigger than usual
  % to show a region outside the target set (where the interval between the
  % upper and lower bounds will be empty).
  g.dim = 1;
  g.min = 2.0 * [ -1 ];
  g.max = 2.0 * [ +1 ];
  g.N = 101;
  g.bdry = @addGhostExtrapolate;
  g = processGrid(g);

  %---------------------------------------------------------------------------
  %% Create initial conditions.
  % A hexagon in velocity versus position space, which translates into
  % maximum (or minimum) position being a piecewise linear function of velocity.
  position_max = min(1.0, 1.5 - g.xs{1});
  position_max = min(position_max, g.xs{1} + 1.5);

  position_min = max(-1.0, g.xs{1} - 1.5);
  position_min = max(position_min, -g.xs{1} - 1.5);
  
  %---------------------------------------------------------------------------
  %% Lower bounds only.
  f = figure;
  plot(position_min, g.xs{1}, 'LineStyle', '--', 'LineWidth', 2, 'Color', 'k');
  axis equal;
  axis([ -1.5, +1.5, g.min, g.max ]);  
  xlabel('position');  ylabel('velocity');
  grid on;

  %---------------------------------------------------------------------------
  %% Upper bounds only.
  f = figure;
  plot(position_max, g.xs{1}, 'LineStyle', '--', 'LineWidth', 2, 'Color', 'k');
  axis equal;
  axis([ -1.5, +1.5, g.min, g.max ]);  
  xlabel('position');  ylabel('velocity');
  grid on;

  %---------------------------------------------------------------------------
  %% Upper and lower bounds, highlighting the target set.
  f = figure;
  
  % First reproduce the curves shown in the first two figures.
  plot(position_min, g.xs{1}, 'LineStyle', '--', 'LineWidth', 1, 'Color', 'k');
  hold on;
  plot(position_max, g.xs{1}, 'LineStyle', '--', 'LineWidth', 1, 'Color', 'k');
  axis equal;
  axis([ -1.5, +1.5, g.min, g.max ]);  
  xlabel('position');  ylabel('velocity');

  % Now highlight the final reach set, including only those portions of the
  % maximum and minimum curves which are feasible (eg: minimum < maximum).
  not_feasible = (position_max < position_min);
  feasible_max = position_max;
  feasible_max(not_feasible) = NaN;
  feasible_min = position_min;
  feasible_min(not_feasible) = NaN;
  plot(feasible_max, g.xs{1}, 'LineStyle', '-', 'LineWidth', 2, 'Color', 'b');
  plot(feasible_min, g.xs{1}, 'LineStyle', '-', 'LineWidth', 2, 'Color', 'b');
  grid on;

