function [ data, g, data0 ] = obliviousImplicit(which_position)
% obliviousImplicit: two vehicles model, but only pursuer has inputs.
%
%   [ data, g, data0 ] = oblivousImplicit(which_position)
%  
% A standard reachability calculation for a restricted version of the two
% identical vehicles model: Assume that the evader has fixed heading and
% velocity, but the pursuer can adjust its linear acceleration and angular
% velocity.
%
% The model is:
%
%     \dot x = -v_e + v_p \cos \psi
%     \dot y = v_p \sin \psi
%     \dot v_p = a_p
%     \dot \psi = \omega_p
%
% where x, y, psi are relative coordinates with the evader fixed at the
% origin facing the positive x-axis, while v_p and v_e are the forward
% speed of the pursuer and evader respectively.  Inputs are pursuer linear
% acceleration and angular velocity.
%
% The target set is a unit box in x - y space, subject to the pursuer's
% linear velocity being within some interval.  We consider pursuer linear
% velocities that are too high or too low to be part of an escape set.
% Note that the evader has no inputs, and so cannot drive the system into
% this escape set consciously.  Instead, the pursuer must use its inputs to
% avoid the escape set.
%
% It would be easy to add evader linear acceleration (and make evader
% linear velocity a state variable) except that it would make visualization
% hard and it is not clear how to handle minimum and maximum speeds for the
% evader.  (Maybe part of the target set?)
%
% There are many options in the function which can only be modifed by
% changing the file.
%
% Input Parameters:
%
%   which_position:  String.  Specifies which relative position states
%   should be included in the calculation.  Pursuer linear velocity and
%   vehicle relative heading are always included, but the relative x and y
%   positions need not both be computed. Optional.  Default is 'all'.
%
%     'all': Include all position variables.  Note that this is a high
%     dimensional computation, and may be very slow / hard to visualize.
%
%     'x_only', 'y_only': Include only the specified relative position variable.
%
% Output Parameters:
%
%   data: Implicit surface function at t_max.
%
%   g: Grid structure on which data was computed.
%
%   data0: Implicit surface function at t_0.

% Copyright 2011 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2011/01/12.
% Subversion tags for version control purposes.
% $Date: 2011-04-12 20:10:31 -0500 (Tue, 12 Apr 2011) $
% $Id: obliviousImplicit.m 582 2011-04-13 01:10:31Z mitchell $

  %---------------------------------------------------------------------------
  % You will see many executable lines that are commented out.
  %   These are included to show some of the options available; modify
  %   the commenting to modify the behavior.

  %---------------------------------------------------------------------------
  %% Make sure we can see the kernel m-files.
  run('addPathToKernel');

  %---------------------------------------------------------------------------
  %% Optional input parameters.
  
  if(nargin < 1)
    which_position = 'all';
  end
  
  %---------------------------------------------------------------------------
  %% PDE solver parameters.

  % Integration parameters.
  tMax = 2.0;                  % End time.
  plotSteps = 9;               % How many intermediate plots to produce?
  t0 = 0;                      % Start time.
  singleStep = 0;              % Plot at each timestep (overrides tPlot).

  % Period at which intermediate plots should be produced.
  tPlot = (tMax - t0) / (plotSteps - 1);

  % How close (relative) do we need to get to tMax to be considered finished?
  small = 100 * eps;

  % Pause after each plot?
  pauseAfterPlot = false;

  % Delete previous plot before showing next?
  deleteLastPlot = true;

  % Plot in separate subplots (set deleteLastPlot = 0 in this case)?
  % For computations in dimension greater than three, the results will be
  % put into different figures since we need the subplots to do slicing to
  % visualize such high dimensional solutions.
  use_subplots = false;

  % Parameters for visualizeLevelSets.
  level = 0;
  displayType = 'surface';
  
  % Parameters for four dimensional visualization.
  visualize_slice_dim = 3;
  visualize_slice_num = 9;

  % Should we only allow the reach set to grow?
  restrict_update = true;

  % Should we mask after every timestep to maintain the restrictions on
  % pursuer velocity?
  use_mask = true;

  % How many steps of reinitialization should we perform each timestep?
  % Set to zero to turn off reinitialization.
  reinit_steps = 0;
  % If reinitialization is used, at what accuracy?
  reinit_accuracy = 'medium'; %#ok<NASGU>
  
  % Spatial approximation scheme.
  derivFunc = @upwindFirstENO2;

  % Temporal approximation scheme.
  integratorFunc = @odeCFL2;

  % Some integration parameters apply to any integrator.
  integratorOptions = odeCFLset('factorCFL', 0.9, 'stats', 'on');
  if(singleStep)
    integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
  end

  % Modify the shape of the target set from a box to a pyramid / hexagon?
  % The implicit formulation can handle the box shape, but the MIE
  % formulation requires that bounds on x and y be continuous functions of
  % psi.  This modification ensures that property.
  modified_box_IC = true;
  % If we do use a modified box, what slope should the modifications be?
  % Larger values are closer to a true box, but the result will behave more
  % like a discontinuity in the MIE version.  Note that this particular
  % number may not be the actual slope used.
  modified_box_slope_factor = 5;
  
  %---------------------------------------------------------------------------
  %% Optional and problem specific parameters.

  % These parameters were chosen primarily to give a reach set which shows
  % significant differentiation over the range of psi and vp, as well as
  % significant difference from the target set.  I also tried to choose
  % parameters which would lead to a relatively well-scaled grid (eg: g.dx
  % does not vary too much from dimension to dimension), and keep the reach
  % set away from the computational boundaries.
  
  % The target "box" in state space.
  params.target_min = [ -1; -1; 1.0; -inf ];
  params.target_max = [ +1; +1; 3.0; +inf ];
  % The mask which represents the set of states (essentially velocities)
  % outside of which the pursuer may not go to achieve the target set.  We
  % implement this bound as a mask because it must be enforced throughout
  % the simulation, rather than just at the initial time.
  params.mask_min = [ -inf; -inf; 1.0; -inf ];
  params.mask_max = [ +inf; +inf; 3.0; +inf ];
  % Bounds on pursuer linear acceleration.
  params.linear_acc_p = [ -0.2, +0.2 ];
  % Bounds on pursuer angular velocity.
  params.angular_vel_p = [ -0.2, +0.2 ];
  % Evader's linear velocity (fixed).
  params.linear_vel_e = +1;
  
  % For clarity of references in the code, create variables mapping from
  % state names to indexes and vice versa.
  switch(which_position)
    case 'all'
      s2i = struct('x', 1, 'y', 2, 'vp', 3, 'psi', 4);
      i2s = { 'x', 'y', 'vp', 'psi' };
      which_state = logical([ 1; 1; 1; 1 ]);
      position_dims = [ 1, 2 ];
    case 'x_only'
      s2i = struct('x', 1, 'y', false, 'vp', 2, 'psi', 3);
      i2s = { 'x', 'vp', 'psi' };
      which_state = logical([ 1; 0; 1; 1 ]);
      position_dims = 1;
    case 'y_only'
      s2i = struct('x', false, 'y', 1, 'vp', 2, 'psi', 3);
      i2s = { 'y', 'vp', 'psi' };
      which_state = logical([ 0; 1; 1; 1 ]);
      position_dims = 1;
    otherwise
      error('Unknown which_position parameter: %s', which_position);
  end
  % Also a structure mapping from state names to the indexes in grid_all,
  % target_min, target_max, mask_min, mask_max.
  all_s2i = struct('x', 1, 'y', 2, 'vp', 3, 'psi', 4);
  
  %---------------------------------------------------------------------------
  %% Create the grid.
  
  % Here are the grid parameters for all possible dimensions.
  grid_all.min = [  -5; -7.5; 0.8;    0 ];
  grid_all.max = [ +10; +7.5; 3.2; 2*pi ];
  % For non-periodic dimensions, we want a round number + 1, while for
  % periodic dimensions we want just the round number.
  grid_all.N = 1 * [ 75; 75; 32; 50 ] + [ 1; 1; 1; 0 ];
  % Adjust location of upper-most node in grid for periodic coordinate.
  grid_all.max(all_s2i.psi) = grid_all.max(all_s2i.psi) * (1 - 1 / grid_all.N(all_s2i.psi));

  % Roughly map the 3D visualization back into a cube shape.
  aspect_ratio_all = [ 5, 5, 1, 2 ];
  
  g.dim = length(i2s);
  g.min = grid_all.min(which_state);
  g.max = grid_all.max(which_state);
  g.N = grid_all.N(which_state);
  g.bdry = cat(1, repmat({ @addGhostExtrapolate }, g.dim - 1, 1), { @addGhostPeriodic });
  g = processGrid(g);

  %---------------------------------------------------------------------------
  %% Create initial and masking conditions.
  
  % Start with a simple box for the target.
  data = shapeRectangleByCorners(g, params.target_min(which_state), params.target_max(which_state));

  if modified_box_IC
    % Modify the target so that the resulting shape can be used as initial
    % conditions in an MIE formulation as well.  In this case, we need to
    % ensure that the maximum and minimum relative positions are continuous
    % functions of psi and vp.  In fact, there is no dependence on psi, so
    % we only need to deal with the dependence on vp for whatever position
    % variables are present in the grid.
    modified_box_normal = zeros(g.dim, 1);
    modified_box_point = zeros(g.dim, 1);
    for i = position_dims
      modified_box_normal(s2i.vp) = +modified_box_slope_factor;
      modified_box_normal(i) = +1;
      modified_box_point(s2i.vp) = params.target_max(all_s2i.vp);
      data = shapeIntersection(data, shapeHyperplane(g, modified_box_normal, modified_box_point));
      modified_box_normal(i) = -1;
      data = shapeIntersection(data, shapeHyperplane(g, modified_box_normal, modified_box_point));
      
      modified_box_normal(s2i.vp) = -modified_box_slope_factor;
      modified_box_normal(i) = +1;
      modified_box_point(s2i.vp) = params.target_min(all_s2i.vp);
      data = shapeIntersection(data, shapeHyperplane(g, modified_box_normal, modified_box_point));
      modified_box_normal(i) = -1;
      data = shapeIntersection(data, shapeHyperplane(g, modified_box_normal, modified_box_point));
    end
  end

  if use_mask
    % Use a box for the mask.  This treatment will work fine for the implicit
    % version, but it is not clear whether it will work for MIE (because it
    % will be a discontinuous mask in that case).
    mask = shapeRectangleByCorners(g, params.mask_min(which_state), params.mask_max(which_state));
  
    % Apply the mask to the initial conditions so that they satisfy the mask.
    % (With properly chosen initial conditions, it should not make any
    % difference.)
    data = max(data, mask);
  end
  
  % If necessary, provide the IC as an output.
  if(nargout > 2)
    data0 = data;
  end

  %---------------------------------------------------------------------------
  %% Set up PDE term that implements the dynamics.

  % A Lax-Friedrichs implementation can compute the optimal feedback
  % inputs although it is likely to be more dissipative numerically and
  % hence less accurate.
  schemeFunc = @termLaxFriedrichs;
  schemeData.derivFunc = derivFunc;
  schemeData.hamFunc = @hamFunc;
  schemeData.partialFunc = @partialFunc;
  schemeData.dissFunc = @artificialDissipationGLF;
  schemeData.grid = g;
  schemeData.params = params;
  schemeData.s2i = s2i;
  schemeData.i2s = i2s;
  
  % Some additional values which we can precompute and hence save time in
  % hamFunc and partialFunc.
  schemeData.dxdt = -params.linear_vel_e + g.xs{s2i.vp} .* cos(g.xs{s2i.psi});
  schemeData.dydt = g.xs{s2i.vp} .* sin(g.xs{s2i.psi});
  schemeData.abs_dxdt = abs(schemeData.dxdt);
  schemeData.abs_dydt = abs(schemeData.dydt);

  %---------------------------------------------------------------------------
  %% Restrict the Hamiltonian so that reachable set only grows.
  % Whatever scheme handles the dynamics must already be completely set up.
  if restrict_update
    innerFunc = schemeFunc;
    innerData = schemeData;
    clear schemeFunc schemeData;

    % Wrap the true Hamiltonian inside the term approximation restriction routine.
    schemeFunc = @termRestrictUpdate;
    schemeData.innerFunc = innerFunc;
    schemeData.innerData = innerData;
    schemeData.positive = 0;
  end

  %---------------------------------------------------------------------------
  %% Apply the mask if necessary.
  if use_mask
    schemeData.maskFunc = @max;
    % Need to convert the mask data into a vector, since it is applied to
    % the vector form of the data array.
    schemeData.maskData = mask(:);
    integratorOptions = odeCFLset(integratorOptions, ...
                                  'postTimestep', @postTimestepMask);    
  end

  %---------------------------------------------------------------------------
  %% Set up reinitialization if necessary.
  if reinit_steps
    % Register reinitialization postTimestep routine.
    integratorOptions = odeCFLset(integratorOptions, 'PostTimestep', @postTimestepReinit); %#ok<UNRCH>
  
    % Add necessary components to schemeData structures.
    schemeData.reinitAccuracy = reinit_accuracy;
    schemeData.reinitSteps = reinit_steps;
  
  end

  %---------------------------------------------------------------------------
  %% Initialize Display
  f = figure;

  if(g.dim <= 3)
    % Visualization in dimensions three or lower is easy.
    
    if use_subplots
      % Set up subplot parameters if necessary.
      rows = ceil(sqrt(plotSteps)); %#ok<UNRCH>
      cols = ceil(plotSteps / rows);
      plotNum = 1;
      subplot(rows, cols, plotNum);
    end

    h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);
    xlabel(i2s{1});  ylabel(i2s{2});  zlabel(i2s{3});
    hold on;
    axis(g.axis);
    daspect(aspect_ratio_all(which_state));
    camlight left;  camlight right;
    
  else
    % Visualization in 4D requires some slicing.
    
    % Set up a subplot grid for the slices.
    rows = ceil(sqrt(visualize_slice_num));
    cols = ceil(visualize_slice_num / rows);
    
    % Set up the index cell array to extract the slices.
    slice_locations = round(linspace(1, g.N(visualize_slice_dim), visualize_slice_num));
    slice_indexes = cell(g.dim, 1);
    for d = 1 : g.dim
      slice_indexes{d} = 1 : g.N(d);
    end
    slice_indexes{visualize_slice_dim} = 1;

    % Extract a grid to use for a single slice; in other words, extract a
    % grid which does not contain the slicing dimension.  Also, extract a
    % list of dimensions which are not the slicing dimension.
    sliced_xs = cell(g.dim - 1, 1);
    not_sliced_dims = zeros(g.dim - 1, 1);
    not_sliced_axis = zeros(2 * (g.dim - 1), 1);
    for d = 1 : g.dim
      if(d < visualize_slice_dim)
        sliced_xs{d} = squeeze(g.xs{d}(slice_indexes{:}));
        not_sliced_dims(d) = d;
        not_sliced_axis(2*d-1) = g.min(d);
        not_sliced_axis(2*d) = g.max(d);
      elseif(d > visualize_slice_dim)
        sliced_xs{d-1} = squeeze(g.xs{d}(slice_indexes{:}));
        not_sliced_dims(d-1) = d;
        not_sliced_axis(2*(d-1)-1) = g.min(d);
        not_sliced_axis(2*(d-1)) = g.max(d);
      end
    end
    % A boolean vector identifying which states are not sliced.
    which_state_not_sliced = true(g.dim, 1);
    which_state_not_sliced(visualize_slice_dim) = false;

    visualize_4D_plot()
  end
  
  drawnow;

  %---------------------------------------------------------------------------
  %% Main loop

  % Loop until tMax (subject to a little roundoff).
  tNow = t0;
  startTime = cputime;
  while(tMax - tNow > small * tMax)

    % Reshape data array into column vector for ode solver call.
    y0 = data(:);

    % How far to step?
    tSpan = [ tNow, min(tMax, tNow + tPlot) ];

    % Take a timestep.
    [ t y ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                    integratorOptions, schemeData);
    tNow = t(end);

    % Get back the correctly shaped data array
    data = reshape(y, g.shape);

    if(pauseAfterPlot)
      % Wait for last plot to be digested.
      pause;
    end

    % Return to last plot if necessary.
    figure(f);
    
    if(g.dim <= 3)
      % Visualization in dimensions three or lower is easy.
      
      % Delete last visualization if necessary.
      if deleteLastPlot
        delete(h);
      end
    
      % Move to next subplot if necessary.
      if use_subplots
        plotNum = plotNum + 1; %#ok<UNRCH>
        subplot(rows, cols, plotNum);
      end
      
      h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t) ]);
    
    else
      % Visualization in 4D requires slicing.
      if(use_subplots || ~deleteLastPlot)
        % Create a new figure for this timestep.
        f = figure;
      else
        clf;
      end
      
      visualize_4D_plot();
    end
    
    drawnow;
  end

  %---------------------------------------------------------------------------
  %% Termination code.  
  % Let the user know how long it all took.
  endTime = cputime;
  fprintf('Total execution time %g seconds\n', endTime - startTime);
  if(g.dim <= 3)
    title(sprintf('Oblivious evader model with implicit reachability at t = %4.1f', tNow));
  else
    % Put a title over all subplots.  If you want to uncomment the following
    % line, get mtit() from Matlab Central.
    %mtit(sprintf('Oblivious evader model with implicit reachability at t = %4.1f', tNow));
  end

  %---------------------------------------------------------------------------
  %---------------------------------------------------------------------------
  %% Subfunction to set up the 4D visualization
  function visualize_4D_plot()
    for slice = 1 : visualize_slice_num
      slice_indexes{visualize_slice_dim} = slice_locations(slice);
      subplot(rows, cols, slice);
      
      % Copied from visualizeLevelSet, although we do not do the conversion
      % to a meshgrid(), so we cannot use isonormals.
      h = patch(isosurface(sliced_xs{:}, squeeze(data(slice_indexes{:})), level));
      set(h, 'FaceColor', 'red', 'EdgeColor', 'none');

      xlabel(i2s{not_sliced_dims(1)});  ylabel(i2s{not_sliced_dims(2)});  zlabel(i2s{not_sliced_dims(3)});
      title(sprintf('%s = %4.2f', i2s{visualize_slice_dim}, g.vs{visualize_slice_dim}(slice_locations(slice))));
      
      lighting phong;
      view(3)
      axis(not_sliced_axis);
      daspect(aspect_ratio_all(which_state_not_sliced));
      camlight left;  camlight right;
    end
  end  
  
end % main function.


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = hamFunc(~, ~, deriv, schemeData)
% hamFunc: analytic Hamiltonian.
%
% hamValue = hamFunc(t, data, deriv, schemeData)
%
% This function implements the hamFunc prototype for the Implicit
% reachability version of the kinematic car model.
%
% Input parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   deriv: Cell vector of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
% Output parameters:
%
%   hamValue: The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .params: Model parameters.
%
%   .s2i: Structure.  Maps from variable names to the corresponding
%   grid dimension (or false if that variable does not appear in the grid).
%
%   .dxdt, .dydt: Precomputed dynamics for position variables.
%
% Ian Mitchell 2011/01/19

  checkStructureFields(schemeData, 'grid', 'params', 's2i', 'dxdt', 'dydt');

  %---------------------------------------------------------------------------
  %% Define some variables to make the dynamics easier to code and more efficient.

  % Unpack the problem data that we use a lot into variables with short
  % names.
  s2i = schemeData.s2i;
  params = schemeData.params;

  %---------------------------------------------------------------------------
  %% Determine the optimal inputs.

  % Both inputs are the right hand sides of integrators, so choice is
  % trivial.
  angular_vel_p = (params.angular_vel_p(1) * (deriv{s2i.psi} > 0) ...
                   + params.angular_vel_p(2) * (deriv{s2i.psi} <= 0));
  linear_acc_p = (params.linear_acc_p(1) * (deriv{s2i.vp} > 0) ...
                   + params.linear_acc_p(2) * (deriv{s2i.vp} <= 0));
                 
  %---------------------------------------------------------------------------
  %% Compute the Hamiltonian.

  % The term which will always be present.
  hamValue = deriv{s2i.psi} .* angular_vel_p + deriv{s2i.vp} .* linear_acc_p;

  if s2i.x
    hamValue = hamValue + deriv{s2i.x} .* schemeData.dxdt;
  end
  
  if s2i.y
    hamValue = hamValue + deriv{s2i.y} .* schemeData.dydt;
  end
  
  % Remember to reverse the dynamics (multiply by -1) since we solve the
  % PDE backward in time.
  hamValue = -hamValue;
  
end % function hamFunc().


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = partialFunc(~, ~, ~, ~, schemeData, dim)
% partialFunc: Hamiltonian partial function.
%
% alpha = partialFunc(t, data, derivMin, derivMax, schemeData, dim)
%
% This function implements the partialFunc prototype for the implicit
% reachability version of the kinematic car model.
%
% It calculates the extrema of the absolute value of the partials of the 
% analytic Hamiltonian with respect to the costate (gradient).
%
% Parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   derivMin: Cell vector of minimum values of the costate (\grad \phi).
%
%   derivMax: Cell vector of maximum values of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
%   dim: Dimension in which the partial derivatives is taken.
%
% Output Parameters:
%
%   alpha: Maximum absolute value of the partial of the Hamiltonian with
%   respect to the costate in dimension dim for the specified range of
%   costate values (O&F equation 5.12). Note that alpha can (and should) be
%   evaluated separately at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .params: Model parameters.
%
%   .i2s: Cell vector.  Maps from grid dimensions to the
%   corresponding variable name.
%
%   .abs_dxdt, .abs_dydt: Absolute value of precomputed dynamics for 
%   position variables.
%
% Ian Mitchell 2011/01/19

  checkStructureFields(schemeData, 'grid', 'params', 'i2s', ...
                                   'abs_dxdt', 'abs_dydt');

  %---------------------------------------------------------------------------
  %% Define some variables to make the dynamics easier to code and more efficient.

  % Unpack the problem data that we use a lot into variables with short
  % names.
  params = schemeData.params;
  
  %---------------------------------------------------------------------------
  %% Determine the alpha.
  switch schemeData.i2s{dim}
      
    case 'psi'
      alpha = max(abs(params.angular_vel_p));

    case 'vp'
      alpha = max(abs(params.linear_acc_p));
      
    case 'x'
      alpha = schemeData.abs_dxdt;

    case 'y'
      alpha = schemeData.abs_dydt;
      
    otherwise
      error('Unknown state: %s', schemeData.i2s{dim});
  end
        
end % function partialFunc().
