function [ data, g, data0 ] = rotatingIntegratorImplicit(which_mu, input_bound)
% rotatingIntegratorImplicit: rotating double integrator.
%
%   [ data, g, data0 ] = rotatingIntegratorImplicit(which_mu)
%  
% This is a standard reachability calculation for the rotating double
% integrator example, which is a variation on the double integrator
%
%     \dot x_1 = -x_2 + \mu(r) u_1
%     \dot x_2 = +x_1 + \mu(r) u_2
%     \dot x_3 = r
%
% where r = sqrt(x_1^2 + x_2^2) and \|u\| \leq input_bound.  Notice that
% the position variable (which is typically plotted horizontally) now
% appears as the final variable and will hence be the vertical coordinate.
%
% There are many options in the function which can only be modifed by
% changing the file.
%
% Input Parameters:
%
%   which_mu: String.  Specifies which mu function to use.  Options are
%
%     'constant': mu(z) = 1.  The system behaves for variables r and x_3 like
%     the first quadrant of the standard double integrator.
%
%     'sin': mu(z) = 2 * sin(4*pi*z).  The optimal input varies with radius and
%     has varying effectiveness.  Default
%
%   input_bound: Double.  Bound on the magnitude of the input (eg:
%   acceleration).  Optional.  Default is 0.5.
%
% Output Parameters:
%
%   data: Implicit surface function at t_max.
%
%   g: Grid structure on which data was computed.
%
%   data0: Implicit surface function at t_0.

% Copyright 2010 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2010/10/14
% Subversion tags for version control purposes.
% $Date: 2011-04-11 10:47:05 -0700 (Mon, 11 Apr 2011) $
% $Id: rotatingIntegratorImplicit.m 581 2011-04-11 17:47:05Z mitchell $

  %---------------------------------------------------------------------------
  % You will see many executable lines that are commented out.
  %   These are included to show some of the options available; modify
  %   the commenting to modify the behavior.

  %---------------------------------------------------------------------------
  %% Make sure we can see the kernel m-files.
  run('addPathToKernel');

  %---------------------------------------------------------------------------
  %% Optional input parameters.

  if(nargin < 1)
    which_mu = 'sin';
  end

  if(nargin < 2)
    input_bound = 0.5;
  end
  
  %---------------------------------------------------------------------------
  %% PDE solver parameters.

  % Integration parameters.
  tMax = 4.0;                  % End time.
  plotSteps = 9;               % How many intermediate plots to produce?
  t0 = 0;                      % Start time.
  singleStep = false;          % Plot at each timestep (overrides tPlot).

  % Period at which intermediate plots should be produced.
  tPlot = (tMax - t0) / (plotSteps - 1);

  % How close (relative) do we need to get to tMax to be considered finished?
  small = 100 * eps;

  % Pause after each plot?
  pauseAfterPlot = false;

  % Delete previous plot before showing next?
  deleteLastPlot = true;

  % Plot in separate subplots (set deleteLastPlot = 0 in this case)?
  useSubplots = false;

  % Parameters for visualizeLevelSets.
  level = 0;
  displayType = 'surface';

  % Should we only allow the reach set to grow?
  restrict_update = true;

  % Spatial approximation scheme.
  derivFunc = @upwindFirstENO2;

  % Temporal approximation scheme.
  integratorFunc = @odeCFL2;

  % Some integration parameters apply to any integrator.
  integratorOptions = odeCFLset('factorCFL', 0.9, 'stats', 'on');
  if(singleStep)
    integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
  end

  %---------------------------------------------------------------------------
  %% Optional and problem specific parameters.

  % What kind of target set?
  target_set_type = 'rectangle'; %#ok<NASGU>
  target_set_type = 'hexagon';
  
  %---------------------------------------------------------------------------
  %% Create the grid.  
  g.dim = 3;
  g.min = [ -1.2; -1.2;   0 ];
  g.max = [ +1.2; +1.2; 1.2 ];
  g.N = [ 60; 60; 30 ] + ones(g.dim, 1);
  g.bdry = @addGhostExtrapolate;
  g = processGrid(g);

  %---------------------------------------------------------------------------
  %% Create initial conditions.
  switch(target_set_type)

    case 'hexagon'
      % For this test, the target set is anything OUTSIDE of a hexagonal
      % shape. We use a hexagon instead of a square so that the maximum
      % position for the target set (x_3) is a continuous function of the
      % velocity (x_1 and x_2).  This continuity is not required for this
      % implicit formulation, but will be important for the MIE formulation.

      % First, construct the hexagon (it is actually more of a truncated
      % pyramid for this 3D problem, but we keep the notation from the 2D
      % version).  Start with a rectangle to create the horizontal top.
      data = shapeRectangleByCorners(g, [ -1; -1; -inf ], [ +1; +1; +1.0 ]);
      data = shapeIntersection(data, shapeHyperplane(g, [ +5;  0; +1 ], [ +1; 0; 0 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [ -5;  0; +1 ], [ -1; 0; 0 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [  0; +5; +1 ], [ 0; +1; 0 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [  0; -5; +1 ], [ 0; -1; 0 ]));

    case 'rectangle'
      % The rectangular target set cannot be done using the MIE formulation,
      % but it works fine for this implicit formulation.
      data = shapeRectangleByCorners(g, [ -1.0; -1.0; -1.0 ], [ +1.0; +1.0; +1.0 ]);

    otherwise
      error('Unknown target_set_type: %s', target_set_type);
  end

  % Then take the complement, since the outside of the set is the actual
  % target for the backwards reachability.
  data = -data;
  % If necessary, provide the IC as an output.
  if(nargout > 2)
    data0 = data;
  end

  %---------------------------------------------------------------------------
  %% Set up PDE term that implements the dynamics.
  
  % A Lax-Friedrichs implementation can compute the optimal feedback
  % inputs although it is likely to be more dissipative numerically and
  % hence less accurate.
  schemeFunc = @termLaxFriedrichs;
  schemeData.derivFunc = derivFunc;
  schemeData.hamFunc = @hamFunc;
  schemeData.partialFunc = @partialFunc;
  schemeData.dissFunc = @artificialDissipationGLF;
  schemeData.grid = g;
  schemeData.input_bound = input_bound;
  schemeData.r = sqrt(g.xs{1}.^2 + g.xs{2}.^2);
  switch(which_mu)
    case 'constant'
      schemeData.mu = 1;
    case 'sin'
      schemeData.mu = 2 * sin(4 * pi * schemeData.r);
    otherwise
      error('Unknown which_mu parameter: %s', which_mu);
  end

  %---------------------------------------------------------------------------
  %% Restrict the Hamiltonian so that reachable set only grows.
  % Whatever scheme handles the dynamics must already be completely set up.
  if(restrict_update)
    innerFunc = schemeFunc;
    innerData = schemeData;
    clear schemeFunc schemeData;

    % Wrap the true Hamiltonian inside the term approximation restriction routine.
    schemeFunc = @termRestrictUpdate;
    schemeData.innerFunc = innerFunc;
    schemeData.innerData = innerData;
    schemeData.positive = 0;
  end

  %---------------------------------------------------------------------------
  %% Initialize Display
  f = figure; %#ok<NASGU>

  % Set up subplot parameters if necessary.
  if(useSubplots)
    rows = ceil(sqrt(plotSteps));
    cols = ceil(plotSteps / rows);
    plotNum = 1;
    subplot(rows, cols, plotNum);
  end

  h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);
  xlabel('velocity 1');  ylabel('velocity 2');  zlabel('position');

  hold on;
  axis(g.axis);
  daspect([ 1 1 1 ]);
  camlight right;  camlight left;
  drawnow;
  
  %---------------------------------------------------------------------------
  %% Loop until tMax (subject to a little roundoff).
  tNow = t0;
  startTime = cputime;
  while(tMax - tNow > small * tMax)

    % Reshape data array into column vector for ode solver call.
    y0 = data(:);

    % How far to step?
    tSpan = [ tNow, min(tMax, tNow + tPlot) ];

    % Take a timestep.
    [ t y ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                    integratorOptions, schemeData);
    tNow = t(end);

    % Get back the correctly shaped data array
    data = reshape(y, g.shape);

    if(pauseAfterPlot)
      % Wait for last plot to be digested.
      pause;
    end

    % Delete last visualization if necessary.
    if(deleteLastPlot)
      delete(h);
    end

    % Move to next subplot if necessary.
    if(useSubplots)
      plotNum = plotNum + 1;
      subplot(rows, cols, plotNum);
    end

    % Create new visualization.
    h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t) ]);
    axis(g.axis);
    daspect([ 1 1 1 ]);

  end

  %---------------------------------------------------------------------------
  %% Termination code.  
  % Let the user know how long it all took.
  endTime = cputime;
  fprintf('Total execution time %g seconds\n', endTime - startTime);

  title(sprintf('Implicit: t = %5.2f, ||u|| = %4.2f', tNow, input_bound));
  
end % main function.


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = hamFunc(~, ~, deriv, schemeData)
% hamFunc: analytic Hamiltonian.
%
% hamValue = hamFunc(t, data, deriv, schemeData)
%
% This function implements the hamFunc prototype for the implicit
% reachability version of the rotating double integrator.
%
% Input parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   deriv: Cell vector of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
% Output parameters:
%
%   hamValue: The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .input_bound: Scalar.  Bound on the norm of the input vector.
%
%   .mu: Array same size as data.  Contains the function mu(x) evaluated at
%   each node in the grid.
%
%   .r: Array same size as data.  Contains sqrt(g.xs{1}.^2 + g.xs{2}.^2).

% Ian Mitchell 2010/10/14

  checkStructureFields(schemeData, 'grid', 'input_bound', 'mu', 'r');

  % Remember to reverse the dynamics (multiply by -1) since we solve the
  % PDE backward in time.
  hamValue = -(-schemeData.grid.xs{2} .* deriv{1} + schemeData.grid.xs{1} .* deriv{2} ...
               + abs(schemeData.mu) * schemeData.input_bound .* sqrt(deriv{1}.^2 + deriv{2}.^2) ...
               + schemeData.r .* deriv{3});

end % function hamFunc().


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = partialFunc(~, ~, ~, ~, schemeData, dim)
% partialFunc: Hamiltonian partial function.
%
% alpha = partialFunc(t, data, derivMin, derivMax, schemeData, dim)
%
% This function implements the partialFunc prototype for the Implicit
% reachability version of the constant linear with input variation on the
% double integrator.
%
% It calculates the extrema of the absolute value of the partials of the 
% analytic Hamiltonian with respect to the costate (gradient).
%
% Parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   derivMin: Cell vector of minimum values of the costate (\grad \phi).
%
%   derivMax: Cell vector of maximum values of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
%   dim: Dimension in which the partial derivatives is taken.
%
% Output Parameters:
%
%   alpha: Maximum absolute value of the partial of the Hamiltonian with
%   respect to the costate in dimension dim for the specified range of
%   costate values (O&F equation 5.12). Note that alpha can (and should) be
%   evaluated separately at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .input_bound: Scalar.  Bound on the norm of the input vector.
%
%   .mu: Array same size as data.  Contains the function mu(x) evaluated at
%   each node in the grid.
%
%   .r: Array same size as data.  Contains sqrt(g.xs{1}.^2 + g.xs{2}.^2).

% Ian Mitchell 2010/10/14

  checkStructureFields(schemeData, 'grid', 'input_bound', 'mu', 'r');

  switch dim
    case 1
      alpha = abs(schemeData.grid.xs{2}) + schemeData.input_bound * abs(schemeData.mu);

    case 2
      alpha = abs(schemeData.grid.xs{1}) + schemeData.input_bound * abs(schemeData.mu);

    case 3
      alpha = schemeData.r;
      
    otherwise
      error('Partials do not exist in dimension %d', dim);
  end
  
end % function partialFunc().
