function [ data, g ] = rotatingIntegratorMIE(which_mu, input_bound)
% rotatingIntegratorMIE: Rotating double integrator.
%
%   [ data ] = rotatingIntegratorMIE(which_mu, input_bound)
%
% This is a reachability calculation using the mixed implicit explicit
% scheme for the rotating double integrator example, which is a variation
% on the double integrator:
%
%     \dot x_1 = -x_2 + \mu(r) u_1
%     \dot x_2 = +x_1 + \mu(r) u_2
%     \dot x_3 = r
%
% where r = sqrt(x_1^2 + x_2^2) and \|u\| \leq input_bound.  Notice that
% the position variable (which is typically plotted horizontally) now
% appears as the final variable and will hence be the vertical coordinate.
%
% Note that for this problem we only compute the upper boundary of the
% reachable set; we are not interested in the lower boundary or regions
% where the upper boundary is negative.
%
% There are many options in the function which can only be modifed by
% changing the file.
%
% Input Parameters:
%
%   which_mu: String.  Specifies which mu function to use.  Options are
%
%     'constant': mu(z) = 1.  The system behaves for variables r and x_3 like
%     the first quadrant of the standard double integrator.
%
%     'sin': mu(z) = 2 * sin(4*pi*z).  The optimal input varies with radius and
%     has varying effectiveness.  Default.
%
%   input_bound: Double.  Bound on the magnitude of the input (eg:
%   acceleration).  Optional.  Default is 0.5.
%
% Output Parameters:
%
%   data: Array, same size as the grid.  Maximum value in the
%   position coordinate which lies within the backward reach set at the end
%   of the integration.
%
%   g: Grid structure on which data was computed.

% Copyright 2009 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 10/02/09
% Subversion tags for version control purposes.
% $Date: 2011-04-11 10:47:05 -0700 (Mon, 11 Apr 2011) $
% $Id: rotatingIntegratorMIE.m 581 2011-04-11 17:47:05Z mitchell $

  %---------------------------------------------------------------------------
  % You will see many executable lines that are commented out.
  %   These are included to show some of the options available; modify
  %   the commenting to modify the behavior.
  
  %---------------------------------------------------------------------------
  %% Make sure we can see the kernel m-files.
  run('addPathToKernel');

  %---------------------------------------------------------------------------
  %% Optional input parameters.

  if(nargin < 1)
    which_mu = 'sin';
  end

  if(nargin < 2)
    input_bound = 0.5;
  end

  %---------------------------------------------------------------------------
  %% PDE solver parameters.

  % Integration parameters.
  tMax = 4.0;                  % End time.
  plotSteps = 9;               % How many intermediate plots to produce?
  t0 = 0;                      % Start time.
  singleStep = 0;              % Plot at each timestep (overrides tPlot).

  % Period at which intermediate plots should be produced.
  tPlot = (tMax - t0) / (plotSteps - 1);

  % How close (relative) do we need to get to tMax to be considered finished?
  small = 100 * eps;

  % Pause after each plot?
  pauseAfterPlot = false;

  % Delete previous plot before showing next?
  deleteLastPlot = true;

  % Plot in separate subplots (set deleteLastPlot = false in this case)?
  useSubplots = false;
  
  % Parameters for visualizeLevelSets.
  level = 0;
  displayType = 'surf';

  % Should we only allow the reach set to grow?
  restrict_update = 1;

  % Spatial approximation scheme.
  derivFunc = @upwindFirstENO2;

  % Temporal approximation scheme.
  integratorFunc = @odeCFL2;

  % Some integration parameters apply to any integrator.
  integratorOptions = odeCFLset('factorCFL', 0.9, 'stats', 'on');
  if(singleStep)
    integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
  end

  %---------------------------------------------------------------------------
  %% Optional and problem specific parameters.

  %---------------------------------------------------------------------------
  %% Create the grid.
  g.dim = 2;
  g.min = [ -1.2; -1.2 ];
  g.max = [ +1.2; +1.2 ];
  g.N = 2 * [ 60; 60 ] + ones(g.dim, 1);
  g.bdry = @addGhostExtrapolate;
  g = processGrid(g);

  %---------------------------------------------------------------------------
  %% Create initial conditions.
  % A hexagon (actually more of a truncated pyramid) in velocity versus
  % position space, which translates into maximum position
  % being a piecewise linear function of the velocities.
  data = min(1.0, 5 * (1.2 - g.xs{1}));
  data = min(data, 5 * (g.xs{1} + 1.2));
  data = min(data, 5 * (1.2 - g.xs{2}));
  data = min(data, 5 * (g.xs{2} + 1.2));
  
  %---------------------------------------------------------------------------
  %% Set up PDE term that implements the nonlinear component of the dynamics.
  
  % A Lax-Friedrichs implementation can compute the optimal feedback
  % inputs although it is likely to be more dissipative numerically and
  % hence less accurate.
  schemeFunc = @termLaxFriedrichs;
  schemeData.derivFunc = derivFunc;
  schemeData.hamFunc = @hamFunc;
  schemeData.partialFunc = @partialFunc;
  schemeData.dissFunc = @artificialDissipationGLF;
  schemeData.grid = g;
  schemeData.input_bound = input_bound;
  schemeData.r = sqrt(g.xs{1}.^2 + g.xs{2}.^2);
  switch(which_mu)
    case 'constant'
      schemeData.mu = 1;
    case 'sin'
      schemeData.mu = 2 * sin(4 * pi * schemeData.r);
    otherwise
      error('Unknown which_mu parameter: %s', which_mu);
  end

  %---------------------------------------------------------------------------
  %% Restrict the Hamiltonian so that reachable set only grows.
  % Whatever scheme handles the dynamics must already be completely set up.
  if restrict_update
    innerFunc = schemeFunc;
    innerData = schemeData;
    clear schemeFunc schemeData;

    % Wrap the true Hamiltonian inside the term approximation restriction routine.
    schemeFunc = @termRestrictUpdate;
    schemeData.innerFunc = innerFunc;
    schemeData.innerData = innerData;
    % Since we are only working on the maximum position, we can choose the
    % direction of the restriction now.  The maximum posiiton should only
    % shrink (since the reach set is everything ABOVE this curve).
    schemeData.positive = 0;
  end

  %---------------------------------------------------------------------------
  %% Initialize Display
  f = figure; %#ok<NASGU>

  % Set up subplot parameters if necessary.
  if useSubplots
    rows = ceil(sqrt(plotSteps)); %#ok<UNRCH>
    cols = ceil(plotSteps / rows);
    subplot(rows, cols, 1);
  end

  % To start with, we'll show both maximum and minimum position values, even
  % though we are going to compute them sequentially.
  h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);
  hold on;
  set(h, 'FaceColor', 'red', 'EdgeColor', 'none');
  xlabel('velocity 1');  ylabel('velocity 2');  zlabel('position');

  hold on;
  axis(g.axis);
  daspect([ 1 1 1 ]);
  camlight right;  camlight left;
  lighting phong;
  drawnow;

  %---------------------------------------------------------------------------
  %% Integrate the HJ PDEs.
  startTime = cputime;
  tNow = t0;
  while(tMax - tNow > small * tMax)

    % Reshape data array into column vector for ode solver call.
    y0 = data(:);

    % How far to step?
    tSpan = [ tNow, min(tMax, tNow + tPlot) ];

    % Take a timestep.
    [ t y ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                    integratorOptions, schemeData);
    tNow = t(end);

    % Get back the correctly shaped data array
    data = reshape(y, g.shape);

    if(pauseAfterPlot)
      % Wait for last plot to be digested.
      pause;
    end

    % Delete last visualization if necessary.
    if(deleteLastPlot)
      delete(h);
    end

    % Move to next subplot if necessary.
    if useSubplots
      plotNum = plotNum + 1; %#ok<UNRCH>
      subplot(rows, cols, plotNum);
    end

    % Create new visualization.
    h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t) ]);
    set(h, 'FaceColor', 'red', 'EdgeColor', 'none');
    drawnow;

  end

  %---------------------------------------------------------------------------
  %% Termination code.  
  % Let the user know how long it all took.
  endTime = cputime;
  fprintf('Total execution time %g seconds\n', endTime - startTime);

  % Delete result from final timestep.
  delete(h);
  
  % Visualize only that portion of the solution that is non-negative.
  data_pos = data;
  data_pos(data < -sqrt(sum(g.dx.^2))) = nan;
  h = visualizeLevelSet(g, data_pos, displayType, level);
  set(h, 'FaceColor', 'red', 'EdgeColor', 'none');
  lighting phong;
  axis([ g.axis, 0, 1.2 ]);
  title(sprintf('MIE: t = %5.2f, ||u|| = %4.2f', tNow, input_bound));
    
end  % Finished main function.


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = hamFunc(~, ~, deriv, schemeData)
% hamFunc: analytic Hamiltonian.
%
% hamValue = hamFunc(t, data, deriv, schemeData)
%
% This function implements the hamFunc prototype for the MIE reachability
% version of the rotating double integrator.
%
% Input parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   deriv: Cell vector of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
% Output parameters:
%
%   hamValue: The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .input_bound: Scalar.  Bound on the norm of the input vector.
%
%   .mu: Array same size as data.  Contains the function mu(x) evaluated at
%   each node in the grid.
%
%   .r: Array same size as data.  Contains sqrt(g.xs{1}.^2 + g.xs{2}.^2).

% Ian Mitchell 2010/10/15

  checkStructureFields(schemeData, 'grid', 'input_bound', 'mu', 'r');

  % Remember to reverse the dynamics (multiply by -1) since we solve the
  % PDE backward in time.
  hamValue = -(-schemeData.grid.xs{2} .* deriv{1} + schemeData.grid.xs{1} .* deriv{2} ...
               + abs(schemeData.mu) * schemeData.input_bound .* sqrt(deriv{1}.^2 + deriv{2}.^2) ...
               - schemeData.r);

end % function hamFunc().


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = partialFunc(~, ~, ~, ~, schemeData, dim)
% partialFunc: Hamiltonian partial function.
%
% alpha = partialFunc(t, data, derivMin, derivMax, schemeData, dim)
%
% This function implements the partialFunc prototype  for the MIE reachability
% version of the rotating double integrator.
%
% It calculates the extrema of the absolute value of the partials of the 
% analytic Hamiltonian with respect to the costate (gradient).
%
% Parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   derivMin: Cell vector of minimum values of the costate (\grad \phi).
%
%   derivMax: Cell vector of maximum values of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
%   dim: Dimension in which the partial derivatives is taken.
%
% Output Parameters:
%
%   alpha: Maximum absolute value of the partial of the Hamiltonian with
%   respect to the costate in dimension dim for the specified range of
%   costate values (O&F equation 5.12). Note that alpha can (and should) be
%   evaluated separately at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .input_bound: Scalar.  Bound on the norm of the input vector.
%
%   .mu: Array same size as data.  Contains the function mu(x) evaluated at
%   each node in the grid.
%
%   .r: Array same size as data.  Contains sqrt(g.xs{1}.^2 + g.xs{2}.^2).

% Ian Mitchell 2010/10/15

  checkStructureFields(schemeData, 'grid', 'input_bound', 'mu', 'r');

  % Very few terms in the Hamiltonian actually depend on the costate, so
  % this partials function is very simple to compute.
  switch dim
    case 1
      alpha = abs(schemeData.grid.xs{2}) + schemeData.input_bound * abs(schemeData.mu);

    case 2
      alpha = abs(schemeData.grid.xs{1}) + schemeData.input_bound * abs(schemeData.mu);

    otherwise
      error('Partials do not exist in dimension %d', dim);
  end
  
end % function partialFunc().
